@@ -1,6 +1,4 @@
 /*
- * $Id: rel-path.c,v 1.1 2005-05-02 19:36:20 keithp Exp $
- *
  * Copyright © 2005 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -26,10 +24,13 @@
 
 #define SIZE 10
 
+static cairo_test_draw_function_t draw;
+
 cairo_test_t test = {
     "rel-path",
     "Tests calls to various relative path functions",
-    SIZE, SIZE
+    SIZE, SIZE,
+    draw
 };
 
 static cairo_test_status_t
@@ -52,5 +53,5 @@ draw (cairo_t *cr, int width, int height)
 int
 main (void)
 {
-    return cairo_test (&test, draw);
+    return cairo_test (&test);
 }
